@@ -49,6 +49,7 @@ fn _memchr_basic(buf: &[u8], c: u8) -> Option<usize> {
 
 #[cfg(any(target_arch = "x86_64", target_arch = "x86"))]
 #[target_feature(enable = "sse2")]
+#[allow(clippy::missing_safety_doc)]
 pub unsafe fn _memchr_sse2(buf: &[u8], c: u8) -> Option<usize> {
     _memchr_sse2_impl(buf, c)
 }
@@ -75,9 +76,12 @@ macro_rules! _unroll_one_chr_16_aa {
 
 #[inline(always)]
 fn _memchr_sse2_impl(buf: &[u8], c: u8) -> Option<usize> {
-    if buf.len() == 0 {
+    if buf.is_empty() {
         return None;
     }
+    if buf[0] == c {
+        return Some(0);
+    }
     //
     let buf_len = buf.len();
     let mut buf_ptr = buf.as_ptr();
@@ -128,7 +132,14 @@ fn _memchr_sse2_impl(buf: &[u8], c: u8) -> Option<usize> {
 
 #[cfg(any(target_arch = "x86_64", target_arch = "x86"))]
 #[target_feature(enable = "avx")]
+#[allow(clippy::missing_safety_doc)]
 pub unsafe fn _memchr_avx(buf: &[u8], c: u8) -> Option<usize> {
+    if buf.is_empty() {
+        return None;
+    }
+    if buf[0] == c {
+        return Some(0);
+    }
     //
     let buf_len = buf.len();
     let mut buf_ptr = buf.as_ptr();
@@ -139,7 +150,7 @@ pub unsafe fn _memchr_avx(buf: &[u8], c: u8) -> Option<usize> {
     let c32: __m256i = _c32_value(c);
     while buf_ptr <= end_ptr.sub(loop_size) {
         let r = _check_c32_uu(buf_ptr, c32, start_ptr);
-        if !r.is_none() {
+        if r.is_some() {
             return r;
         }
         buf_ptr = buf_ptr.add(loop_size);
@@ -147,11 +158,7 @@ pub unsafe fn _memchr_avx(buf: &[u8], c: u8) -> Option<usize> {
     //
     let next_idx = plus_offset_from(buf_ptr, start_ptr);
     let r = _memchr_sse2(&buf[next_idx..], c);
-    if let Some(pos) = r {
-        Some(pos + next_idx)
-    } else {
-        None
-    }
+    r.map(|pos| pos + next_idx)
 }
 
 #[inline(always)]
